package com.yts.android.YTSBlueH;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.concurrent.locks.ReentrantReadWriteLock;

import android.bluetooth.BluetoothSocket;

abstract class ConnectedThread extends Thread {

	static final int BUFFER_SIZE = 1024;
	final BluetoothSocket mSocket;
	final InputStream mInStream;
	final OutputStream mOutStream;
	final ReentrantReadWriteLock mLock;
	boolean mClosed;

	ConnectedThread(BluetoothSocket socket) {
		mSocket = socket;
		InputStream inStream = null;
		OutputStream outStream = null;

		try {
			inStream = socket.getInputStream();
			outStream = socket.getOutputStream();
		} catch (IOException e) {
		}

		mInStream = inStream;
		mOutStream = outStream;
		mLock = new ReentrantReadWriteLock();
	}

	@Override
	public final void run() {
		byte[] buffer = new byte[BUFFER_SIZE];
		int bytes;

		for (;;) {
			try {
				bytes = mInStream.read(buffer);
				onReceive(buffer, bytes);
			} catch (IOException e) {
				mLock.readLock().lock();

				try {
					if (mClosed) {
						break;
					}
				} finally {
					mLock.readLock().unlock();
				}

				connectionLost(e);
				cancel();
				break;
			}
		}
	}

	final void cancel() {
		mLock.readLock().lock();

		try {
			if (mClosed) {
				return;
			}
		} finally {
			mLock.readLock().unlock();
		}

		mLock.writeLock().lock();

		try {
			mClosed = true;
		} finally {
			mLock.writeLock().unlock();
		}

		try {
			mSocket.close();
		} catch (IOException e) {
		}
	}

	final void write(byte[] bytes) {
		try {
			mOutStream.write(bytes);
		} catch (IOException e) {
			connectionLost(e);
			cancel();
		}
	}

	abstract void connectionLost(IOException error);

	abstract void onReceive(byte[] buffer, int bytes);
}
