package com.yts.android.YTSBlueH;

import static com.yts.android.YTSBlueH.Constants.HEADER_INDEX;
import static com.yts.android.YTSBlueH.Constants.HID_CONTROL_CHANNEL;
import static com.yts.android.YTSBlueH.Constants.HID_INTERRUPT_CHANNEL;
import static com.yts.android.YTSBlueH.Constants.HUMAN_INTERFACE_DEVICE_PROFILE;

import java.io.IOException;
import java.util.Arrays;
import java.util.Set;
import java.util.UUID;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.os.ParcelUuid;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

public final class YTSBlueH extends Activity {

    private TextView mResp;
    
	final class ControlConnectedThrad extends ConnectedThread {

		ControlConnectedThrad(BluetoothSocket socket) {
			super(socket);
		}

	    
		@Override
		void connectionLost(IOException error) {
			Message msg = mHandler.obtainMessage(MESSAGE_CONNECTION_LOST, error
					.toString());
			msg.sendToTarget();
		}

		@Override
		void onReceive(byte[] buffer, int bytes) {
			Message msg = mHandler.obtainMessage(MESSAGE_CONTROL_READ, 0,
					bytes, buffer);
			msg.sendToTarget();

			// for debug
			byte[] data = new byte[bytes];
			System.arraycopy(buffer, 0, data, 0, bytes);
			String reprData = reprBuffer(data);
			Message dmsg = mHandler.obtainMessage(MESSAGE_SYS_OUT, "control:"
					+ reprData);
			dmsg.sendToTarget();
		}

	}

	final class ControlConnectThread extends L2CAPConnectThread {

		ControlConnectThread(BluetoothDevice remoteDevice) {
			super(remoteDevice, HID_CONTROL_CHANNEL);
		}

		@Override
		void connectionFailure(IOException cause) {
			Message msg = mHandler.obtainMessage(MESSAGE_CONNECTION_FAIL, cause
					.toString());
			msg.sendToTarget();
			reconnect();
		}

		@Override
		void manageConnectedSocket(BluetoothSocket socket) {
			BluetoothDevice remDev = socket.getRemoteDevice();
			String name = remDev.getName();
			String addr = remDev.getAddress();
			Message msg = mHandler.obtainMessage(MESSAGE_CONTROL_CONNECTED,
					(name + "\n" + addr));
			msg.sendToTarget();
			mControlConnectedThread = new ControlConnectedThrad(socket);
			mControlConnectedThread.start();
//			mControlConnectedThread.write(new byte[] { 0x71 });
			mInterruptConnectThread = new InterruptConnectThread(mRemoteDevice);
			mInterruptConnectThread.start();
		}
	}

	final class InterruptConnectedThrad extends ConnectedThread {

		InterruptConnectedThrad(BluetoothSocket socket) {
			super(socket);
		}

		@Override
		void connectionLost(IOException error) {
			Message msg = mHandler.obtainMessage(MESSAGE_CONNECTION_LOST, error
					.toString());
			msg.sendToTarget();
		}

		@Override
		void onReceive(byte[] buffer, int bytes) {
			Message msg = mHandler
					.obtainMessage(MESSAGE_READ, 0, bytes, buffer);
			msg.sendToTarget();

			// for debug
			byte[] data = new byte[bytes];
			System.arraycopy(buffer, 0, data, 0, bytes);
			String reprData = reprBuffer(data);
			Message dmsg = mHandler.obtainMessage(MESSAGE_SYS_OUT, reprData);
			dmsg.sendToTarget();
		}
	}

	final class InterruptConnectThread extends L2CAPConnectThread {

		InterruptConnectThread(BluetoothDevice remoteDevice) {
			super(remoteDevice, HID_INTERRUPT_CHANNEL);
		}

		@Override
		void connectionFailure(IOException cause) {
			Message msg = mHandler.obtainMessage(MESSAGE_CONNECTION_FAIL, cause
					.toString());
			msg.sendToTarget();
			reconnect();
		}

		@Override
		void manageConnectedSocket(BluetoothSocket socket) {
			BluetoothDevice remDev = socket.getRemoteDevice();
			String name = remDev.getName();
			String addr = remDev.getAddress();
			Message msg = mHandler.obtainMessage(MESSAGE_CONNECTED, (name
					+ "\n" + addr));
			msg.sendToTarget();
			mInterruptConnectedThread = new InterruptConnectedThrad(socket);
			mInterruptConnectedThread.start();
		}
	}

	final class MyHandler extends Handler {

		@Override
		public void handleMessage(Message msg) {
			String reprObj;
			switch (msg.what) {
			case MESSAGE_CONTROL_READ:
				break;
			case MESSAGE_CONTROL_CONNECTED:
//				reprObj = (String) msg.obj;
//				showToast("connected control:\n" + reprObj);
				break;
			case MESSAGE_READ:
				byte[] buffer = (byte[]) msg.obj;
				int bytes = msg.arg2;

				if (bytes == 0) {
					break;
				}
//Received data to Text				
				String resultString;
				resultString = new String(buffer);
				mResp.setText(resultString.substring(0, bytes));
				break;
				
			case MESSAGE_CONNECTED:			
//				reprObj = (String) msg.obj;
//				showToast("connected:\n" + reprObj);
				mControlConnectedThread.write(new byte[] { (byte) 0x90, (byte)0x00 });

				mResp.setText("Connected to YTS");
				break;
			case MESSAGE_CONNECTION_LOST:
				reprObj = (String) msg.obj;
				showToast("connection lost:\n" + reprObj);
				break;
			case MESSAGE_CONNECTION_FAIL:
				reprObj = (String) msg.obj;
				showToast("faild to connect:\n" + reprObj);
				break;
			case MESSAGE_TOAST:
				reprObj = (String) msg.obj;
				showToast(reprObj);
				break;
			case MESSAGE_SYS_OUT:
				reprObj = (String) msg.obj;
				System.out.println(reprObj);
				break;
			}
		}
	}

	static final int MESSAGE_CONNECTION_FAIL = 5;
	static final int MESSAGE_CONNECTION_LOST = -1;
	static final int MESSAGE_WRITE = 2;
	static final int MESSAGE_READ = 1;
	static final int MESSAGE_CONNECTED = 0;
	static final int MESSAGE_CONTROL_CONNECTED = 10;
	static final int MESSAGE_CONTROL_READ = 11;
	static final int MESSAGE_SYS_OUT = 3;
	static final int MESSAGE_TOAST = 9;

	final Handler mHandler = new MyHandler();
	BluetoothAdapter mLocalDevice;
	BluetoothDevice mRemoteDevice;
	L2CAPConnectThread mInterruptConnectThread;
	ConnectedThread mInterruptConnectedThread;
	L2CAPConnectThread mControlConnectThread;
	ConnectedThread mControlConnectedThread;
	EditText mEditText;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		//Set reference to EditText widget		
		mEditText = (EditText) findViewById(R.id.editText);
		mLocalDevice = BluetoothAdapter.getDefaultAdapter();
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		cancelThread();
	}

	@Override
	protected void onStart() {
		super.onStart();

		if (mRemoteDevice == null) {
			mRemoteDevice = obtainRemoteDevice();
		}

		if (mRemoteDevice == null) {
			Message msg = mHandler.obtainMessage(MESSAGE_TOAST,
					"NO HID DEVICE FOUND");
			msg.sendToTarget();
			return;
		}
		mResp = (TextView) findViewById(R.id.textView1);
		
		cancelThread();
		mControlConnectThread = new ControlConnectThread(mRemoteDevice);
		mControlConnectThread.start();
	}

	void cancelThread() {
		if (mControlConnectThread != null) {
			mControlConnectThread.cancel();
		}

		if (mControlConnectedThread != null) {
			mControlConnectedThread.cancel();
		}

		if (mInterruptConnectThread != null) {
			mInterruptConnectThread.cancel();
		}

		if (mInterruptConnectedThread != null) {
			mInterruptConnectedThread.cancel();
		}
	}

	boolean hasHIDProfile(BluetoothDevice remDev) {
		IBluetoothDevice delegate = DelegateFactory.create(
				IBluetoothDevice.class, remDev);
		ParcelUuid[] uuids = delegate.getUuids();

		for (ParcelUuid puuid : uuids) {
			UUID uuid = puuid.getUuid();

			if (uuid.equals(HUMAN_INTERFACE_DEVICE_PROFILE)) {
				return true;
			}
		}

		return false;
	};

	BluetoothDevice obtainRemoteDevice() {
		Set<BluetoothDevice> pairdDevices = mLocalDevice.getBondedDevices();

		for (BluetoothDevice remDev : pairdDevices) {
			boolean hid = hasHIDProfile(remDev);

			if (hid) {
				return remDev;
			}
		}

		return null;
	}

	void reconnect() {
		try {
			Thread.sleep(500);
		} catch (InterruptedException e) {
		}

		cancelThread();
		mControlConnectThread = new ControlConnectThread(mRemoteDevice);
		mControlConnectThread.start();
	}

	String reprBuffer(byte[] buf) {
		String[] repr = new String[buf.length];
		repr[HEADER_INDEX] = Integer.toHexString((buf[HEADER_INDEX] & 0xff));

		for (int i = 1, len = buf.length; i < len; ++i) {
			repr[i] = Integer.toString((buf[i] & 0xff));
		}

		return Arrays.toString(repr);
	}

	void showToast(String text) {
		Toast t = Toast.makeText(this, text, Toast.LENGTH_SHORT);
		t.show();
	}
	
	public void SetButtonOnClick(View v){
//send data to PIC
		
		byte[] asciiCodes;
        String text = mEditText.getText().toString();
        asciiCodes=text.getBytes();

		mInterruptConnectedThread.write(asciiCodes);
	} 
}